@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2016 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2017 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
  * @brief Platform QoS API and data structure definition
  *
  * API from this file is implemented by the following:
- * host_cap.c, host_allocation.c, host_monitoring.c and utils.c
+ * cap.c, allocation.c, monitoring.c and utils.c
  */
 
 #ifndef __PQOS_H__
@@ -57,7 +57,7 @@ extern "C" {
  * =======================================
  */
 
-#define PQOS_VERSION       105          /**< version 1.05 */
+#define PQOS_VERSION       10200        /**< version 1.2.0 */
 #define PQOS_MAX_L3CA_COS  16           /**< 16 x COS */
 #define PQOS_MAX_L2CA_COS  16           /**< 16 x COS */
 
@@ -73,6 +73,15 @@ extern "C" {
 #define PQOS_RETVAL_INIT         4      /**< initialization error */
 #define PQOS_RETVAL_TRANSPORT    5      /**< transport error */
 #define PQOS_RETVAL_PERF_CTR     6      /**< performance counter error */
+#define PQOS_RETVAL_BUSY         7      /**< resource busy error */
+
+/*
+ * =======================================
+ * Interface values
+ * =======================================
+ */
+#define PQOS_INTER_MSR            0      /**< MSR */
+#define PQOS_INTER_OS             1      /**< OS */
 
 /*
  * =======================================
@@ -89,10 +98,33 @@ enum pqos_cdp_config {
 
 /**
  * PQoS library configuration structure
+ *
+ * @param fd_log file descriptor to be used as library log
+ * @param callback_log pointer to an application callback function
+ *         void *       - An application context - it can point to a structure
+ *                        or an object that an application may find useful
+ *                        when receiving the callback
+ *         const size_t - the size of the log message
+ *         const char * - the log message
+ * @param context_log application specific data that is provided
+ *                    to the callback function. It can be NULL if application
+ *                    doesn't require it.
+ * @param verbose logging options
+ *         LOG_VER_SILENT         - no messages
+ *         LOG_VER_DEFAULT        - warning and error messages
+ *         LOG_VER_VERBOSE        - warning, error and info messages
+ *         LOG_VER_SUPER_VERBOSE  - warning, error, info and debug messages
+ *
+ * @param interface preference
+ *         PQOS_INTER_MSR      - MSR interface or nothing
+ *         PQOS_INTER_OS       - OS interface or nothing
  */
 struct pqos_config {
-        int fd_log;     /**< file descriptor for library log messages */
-        int verbose;    /**< if true increases library verbosity level */
+        int fd_log;
+        void (*callback_log)(void *, const size_t, const char *);
+        void *context_log;
+        int verbose;
+        int interface;
 };
 
 /**
@@ -104,6 +136,8 @@ struct pqos_config {
  *
  * @return Operation status
  * @retval PQOS_RETVAL_OK on success
+ * @note   If you require system wide interface enforcement you can do so by
+ *         setting the "RDT_IFACE" environment variable.
  */
 int pqos_init(const struct pqos_config *config);
 
@@ -128,11 +162,13 @@ int pqos_fini(void);
  * - monitoring capability
  * - L3 cache allocation capability
  * - L2 cache allocation capability
+ * - Memory Bandwidth Allocation
  */
 enum pqos_cap_type {
         PQOS_CAP_TYPE_MON = 0,          /**< QoS monitoring */
         PQOS_CAP_TYPE_L3CA,             /**< L3/LLC cache allocation */
         PQOS_CAP_TYPE_L2CA,             /**< L2 cache allocation */
+        PQOS_CAP_TYPE_MBA,              /**< Memory Bandwidth Allocation */
         PQOS_CAP_TYPE_NUMOF
 };
 
@@ -162,6 +198,17 @@ struct pqos_cap_l2ca {
         uint64_t way_contention;        /**< ways contention bit mask */
 };
 
+/**
+ * Memory Bandwidth Allocation capability structure
+ */
+struct pqos_cap_mba {
+        unsigned mem_size;              /**< byte size of the structure */
+        unsigned num_classes;           /**< number of classes of service */
+        unsigned throttle_max;          /**< the max MBA can be throttled */
+        unsigned throttle_step;         /**< MBA granularity */
+        int is_linear;                  /**< the type of MBA linear/nonlinear */
+};
+
 /**
  * Available types of monitored events
  * (matches CPUID enumeration)
@@ -172,6 +219,8 @@ enum pqos_mon_event {
         PQOS_MON_EVENT_TMEM_BW = 4,     /**< Total memory bandwidth */
         PQOS_MON_EVENT_RMEM_BW = 8,     /**< Remote memory bandwidth
                                            (virtual event) */
+        RESERVED1 = 0x1000,
+        RESERVED2 = 0x2000,
         PQOS_PERF_EVENT_LLC_MISS = 0x4000, /**< LLC misses */
         PQOS_PERF_EVENT_IPC    = 0x8000, /**< instructions per clock */
 };
@@ -193,7 +242,7 @@ struct pqos_monitor {
                                            this event */
         uint32_t scale_factor;          /**< factor to scale RMID value
                                            to bytes */
-        unsigned pid_support;           /**< flag to show if PID monitoring
+        int os_support;                 /**< flag to show if OS monitoring
                                            is supported */
 };
 
@@ -211,10 +260,12 @@ struct pqos_cap_mon {
  */
 struct pqos_capability {
         enum pqos_cap_type type;
+        int os_support;                 /**< OS support presence */
         union {
                 struct pqos_cap_mon *mon;
                 struct pqos_cap_l3ca *l3ca;
                 struct pqos_cap_l2ca *l2ca;
+                struct pqos_cap_mba *mba;
                 void *generic_ptr;
         } u;
 };
@@ -342,8 +393,9 @@ struct pqos_mon_data {
         int *fds_llc;
         int *fds_mbl;
         int *fds_mbt;
-        int **fds_ipc;
-        int **fds_misses;
+        int *fds_inst;
+        int *fds_cyc;
+        int *fds_llc_misses;
 
         /**
          * Core specific section
@@ -393,6 +445,9 @@ int pqos_mon_assoc_get(const unsigned lcore,
  *
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
+ *
+ * @note As of Kernel 4.10, Intel(R) RDT perf results per core are found to
+ *       be incorrect.
  */
 int pqos_mon_start(const unsigned num_cores,
                    const unsigned *cores,
@@ -468,11 +523,42 @@ int pqos_alloc_assoc_set(const unsigned lcore,
 int pqos_alloc_assoc_get(const unsigned lcore,
                          unsigned *class_id);
 
+/**
+ * @brief OS interface to associate \a task
+ *        with given class of service
+ *
+ * @param [in] task task ID to be associated
+ * @param [in] class_id class of service
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_alloc_assoc_set_pid(const pid_t task,
+                             const unsigned class_id);
+
+/**
+ * @brief OS interface to read association
+ *        of \a task with class of service
+ *
+ * @param [in] task ID to find association
+ * @param [out] class_id class of service
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_alloc_assoc_get_pid(const pid_t task,
+                             unsigned *class_id);
+
 /**
  * @brief Assign first available COS to cores in \a core_array
  *
  * While searching for available COS take technologies it is intended to use
  * with into account.
+ * Note on \a technology and \a core_array selection:
+ * - if L2 CAT technology is requested then cores need to belong to
+ *   one L2 cluster (same L2ID)
+ * - if only L3 CAT is requested then cores need to belong to one socket
+ * - if only MBA is selected then cores need to belong to one socket
  *
  * @param [in] technology bit mask selecting technologies
  *             (1 << enum pqos_cap_type)
@@ -499,13 +585,50 @@ int pqos_alloc_release(const unsigned *core_array,
                        const unsigned core_num);
 
 /**
- * @brief Resets configuration of cache allocation technology
+ * @brief Assign first available COS to tasks in \a task_array
+ *        Searches all COS directories from highest to lowest
  *
- * Reverts CAT state to the one after reset:
+ * While searching for available COS take technologies it is intended to use
+ * with into account.
+ * Note on \a technology parameter:
+ * - this parameter is currently reserved for future use
+ * - resctrl (Linux interface) will only provide the highest class id common
+ *   to all supported technologies
+ *
+ * @param [in] technology bit mask selecting technologies
+ *             (1 << enum pqos_cap_type)
+ * @param [in] task_array list of task ids
+ * @param [in] task_num number of task ids in the \a task_array
+ * @param [out] class_id place to store reserved COS id
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_alloc_assign_pid(const unsigned technology,
+                          const pid_t *task_array,
+                          const unsigned task_num,
+                          unsigned *class_id);
+
+/**
+ * @brief Reassign tasks in \a task_array to default COS#0
+ *
+ * @param [in] task_array list of task ids
+ * @param [in] task_num number of task ids in the \a task_array
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_alloc_release_pid(const pid_t *task_array,
+                           const unsigned task_num);
+
+/**
+ * @brief Resets configuration of allocation technologies
+ *
+ * Reverts CAT/MBA state to the one after reset:
  * - all cores associated with COS0
- * - all COS are set to give access to all cache ways
+ * - all COS are set to give access to entire resource
  *
- * As part of CAT reset CDP reconfiguration can be performed.
+ * As part of allocation reset CDP reconfiguration can be performed.
  * This can be requested via \a l3_cdp_cfg.
  *
  * @param [in] l3_cdp_cfg requested L3 CAT CDP config
@@ -556,7 +679,7 @@ int pqos_l3ca_set(const unsigned socket,
  *
  * @param [in] socket CPU socket id
  * @param [in] max_num_ca maximum number of classes of service
- *            that can be accommodated at \a ca
+ *             that can be accommodated at \a ca
  * @param [out] num_ca number of classes of service read into \a ca
  * @param [out] ca table with read classes of service
  *
@@ -568,6 +691,19 @@ int pqos_l3ca_get(const unsigned socket,
                   unsigned *num_ca,
                   struct pqos_l3ca *ca);
 
+/**
+ * @brief Get minimum number of bits which must be set in L3 way mask when
+ *        updating a class of service
+ *
+ * @param [out] min_cbm_bits minimum number of bits that must be set
+ *
+ * @return Operational status
+ * @retval PQOS_RETVAL_OK on success
+ * @retval PQOS_RETVAL_RESOURCE if unable to determine
+ */
+int pqos_l3ca_get_min_cbm_bits(unsigned *min_cbm_bits);
+
+
 /*
  * =======================================
  * L2 cache allocation
@@ -583,23 +719,23 @@ struct pqos_l2ca {
 };
 
 /**
- * @brief Sets classes of service defined by \a ca on \a socket
+ * @brief Sets classes of service defined by \a ca on \a l2id
  *
- * @param [in] socket CPU socket id
+ * @param [in] l2id unique L2 cache identifier
  * @param [in] num_cos number of classes of service at \a ca
  * @param [in] ca table with class of service definitions
  *
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
  */
-int pqos_l2ca_set(const unsigned socket,
+int pqos_l2ca_set(const unsigned l2id,
                   const unsigned num_cos,
                   const struct pqos_l2ca *ca);
 
 /**
- * @brief Reads classes of service from \a socket
+ * @brief Reads classes of service from \a l2id
  *
- * @param [in] socket CPU socket id
+ * @param [in] l2id unique L2 cache identifier
  * @param [in] max_num_ca maximum number of classes of service
  *             that can be accommodated at \a ca
  * @param [out] num_ca number of classes of service read into \a ca
@@ -608,11 +744,70 @@ int pqos_l2ca_set(const unsigned socket,
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
  */
-int pqos_l2ca_get(const unsigned socket,
+int pqos_l2ca_get(const unsigned l2id,
                   const unsigned max_num_ca,
                   unsigned *num_ca,
                   struct pqos_l2ca *ca);
 
+/**
+ * @brief Get minimum number of bits which must be set in L2 way mask when
+ *        updating a class of service
+ *
+ * @param [out] min_cbm_bits minimum number of bits that must be set
+ *
+ * @return Operational status
+ * @retval PQOS_RETVAL_OK on success
+ * @retval PQOS_RETVAL_RESOURCE if unable to determine
+ */
+int pqos_l2ca_get_min_cbm_bits(unsigned *min_cbm_bits);
+
+/*
+ * =======================================
+ * Memory Bandwidth Allocation
+ * =======================================
+ */
+
+/**
+ * MBA class of service data structure
+ */
+struct pqos_mba {
+        unsigned class_id;      /**< class of service */
+        unsigned mb_rate;       /**< valve open rate (VOR) in percentage */
+};
+
+/**
+ * @brief Sets classes of service defined by \a mba on \a socket
+ *
+ * @param [in]  socket CPU socket id
+ * @param [in]  num_cos number of classes of service at \a ca
+ * @param [in]  requested table with class of service definitions
+ * @param [out] actual table with class of service definitions
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_mba_set(const unsigned socket,
+                 const unsigned num_cos,
+                 const struct pqos_mba *requested,
+                 struct pqos_mba *actual);
+
+/**
+ * @brief Reads MBA from \a socket
+ *
+ * @param [in]  socket CPU socket id
+ * @param [in]  max_num_cos maximum number of classes of service
+ *              that can be accommodated at \a mba_tab
+ * @param [out] num_cos number of classes of service read into \a mba_tab
+ * @param [out] mba_tab table with read classes of service
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_mba_get(const unsigned socket,
+                 const unsigned max_num_cos,
+                 unsigned *num_cos,
+                 struct pqos_mba *mba_tab);
+
 /*
  * =======================================
  * Utility API
@@ -632,13 +827,26 @@ unsigned *
 pqos_cpu_get_sockets(const struct pqos_cpuinfo *cpu,
                      unsigned *count);
 
+/**
+ * @brief Retrieves L2 id's from cpu info structure
+ *
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
+ * @param [out] count place to store actual number of L2 id's returned
+ *
+ * @return Allocated array of size \a count populated with L2 id's
+ * @retval NULL on error
+ */
+unsigned *
+pqos_cpu_get_l2ids(const struct pqos_cpuinfo *cpu,
+                   unsigned *count);
+
 /**
  * @brief Creates list of cores belonging to given L3 cluster
  *
  * Function allocates memory for the core list that needs to be freed by
  * the caller.
  *
- * @param [in] cpu CPU topology
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
  * @param [in] l3_id L3 cluster ID
  * @param [out] count place to put number of cores found
  *
@@ -652,7 +860,7 @@ pqos_cpu_get_cores_l3id(const struct pqos_cpuinfo *cpu, const unsigned l3_id,
 /**
  * @brief Retrieves core id's from cpu info structure for \a socket
  *
- * @param [in] cpu CPU information structure from cpu info module
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
  * @param [in] socket CPU socket id to enumerate
  * @param [out] count place to store actual number of core id's returned
  *
@@ -665,9 +873,33 @@ pqos_cpu_get_cores(const struct pqos_cpuinfo *cpu,
                    unsigned *count);
 
 /**
+ * @brief Retrieves task id's from resctrl task file for a given COS
+ *
+ * @param [in] class_id Class of Service ID
+ * @param [out] count place to store actual number of task id's returned
+ *
+ * @return Allocated task id array
+ * @retval NULL on error
+ */
+unsigned *
+pqos_pid_get_pid_assoc(const unsigned class_id, unsigned *count);
+
+/**
+ * @brief Retrieves core information from cpu info structure for \a lcore
+ *
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
+ * @param [in] lcore logical core ID to retrieve information for
+ *
+ * @return Pointer to core information structure
+ * @retval NULL on error
+ */
+const struct pqos_coreinfo *
+pqos_cpu_get_core_info(const struct pqos_cpuinfo *cpu, unsigned lcore);
+
+ /**
  * @brief Retrieves one core id from cpu info structure for \a socket
  *
- * @param [in] cpu CPU information structure from cpu info module
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
  * @param [in] socket CPU socket id to enumerate
  * @param [out] lcore place to store returned core id
  *
@@ -679,10 +911,25 @@ pqos_cpu_get_one_core(const struct pqos_cpuinfo *cpu,
                       const unsigned socket,
                       unsigned *lcore);
 
+/**
+ * @brief Retrieves one core id from cpu info structure for \a l2id
+ *
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
+ * @param [in] l2id unique L2 cache identifier
+ * @param [out] lcore place to store returned core id
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int
+pqos_cpu_get_one_by_l2id(const struct pqos_cpuinfo *cpu,
+                         const unsigned l2id,
+                         unsigned *lcore);
+
 /**
  * @brief Verifies if \a core is a valid logical core id
  *
- * @param [in] cpu CPU information structure from cpu info module
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
  * @param [in] lcore logical core id
  *
  * @return Operation status
@@ -695,7 +942,7 @@ pqos_cpu_check_core(const struct pqos_cpuinfo *cpu,
 /**
  * @brief Retrieves socket id for given logical core id
  *
- * @param [in] cpu CPU information structure from cpu info module
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
  * @param [in] lcore logical core id
  * @param [out] socket location to store socket id at
  *
@@ -710,7 +957,7 @@ pqos_cpu_get_socketid(const struct pqos_cpuinfo *cpu,
 /**
  * @brief Retrieves monitoring cluster id for given logical core id
  *
- * @param [in] cpu CPU information structure from cpu info module
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
  * @param [in] lcore logical core id
  * @param [out] cluster location to store cluster id at
  *
@@ -784,6 +1031,21 @@ int
 pqos_l2ca_get_cos_num(const struct pqos_cap *cap,
                       unsigned *cos_num);
 
+/**
+ * @brief Retrieves number of memory B/W allocation classes of service from
+ *        \a cap structure.
+ *
+ * @param [in] cap platform QoS capabilities structure
+ *                 returned by \a pqos_cap_get
+ * @param [out] cos_num place to store number of classes of service
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int
+pqos_mba_get_cos_num(const struct pqos_cap *cap,
+                     unsigned *cos_num);
+
 /**
  * @brief Retrieves CDP status
  *
