@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2016 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2017 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,7 @@
 #include "profiles.h"
 #include "monitor.h"
 #include "alloc.h"
+#include "cap.h"
 
 /**
  * Default CDP configuration option - don't enforce on or off
@@ -83,15 +84,30 @@ static char *sel_allocation_profile = NULL;
 static int sel_verbose_mode = 0;
 
 /**
- * Reset CAT configuration
+ * Reset allocation configuration
  */
-static int sel_reset_CAT = 0;
+static int sel_reset_alloc = 0;
 
 /**
  * Enable showing cache allocation settings
  */
 static int sel_show_allocation_config = 0;
 
+/**
+ * Enable displaying supported RDT capabilities
+ */
+static int sel_display = 0;
+
+/**
+ * Enable displaying supported RDT capabilities in verbose mode
+ */
+static int sel_display_verbose = 0;
+
+/**
+ * Selected library interface
+ */
+int sel_interface = PQOS_INTER_MSR;
+
 /**
  * @brief Function to check if a value is already contained in a table
  *
@@ -280,12 +296,12 @@ selfn_super_verbose_mode(const char *arg)
 }
 
 /**
- * @brief Sets CAT reset flag
+ * @brief Sets allocation reset flag
  *
  * @param [in] arg optional configuration string
  *             if NULL or zero length  then configuration check is skipped
  */
-static void selfn_reset_cat(const char *arg)
+static void selfn_reset_alloc(const char *arg)
 {
         if (arg != NULL && (strlen(arg) > 0)) {
                 const struct {
@@ -303,12 +319,13 @@ static void selfn_reset_cat(const char *arg)
                                 break;
 
                 if (i >= DIM(patterns)) {
-                        printf("Unrecognized '%s' CAT reset option!\n", arg);
+                        printf("Unrecognized '%s' allocation "
+                               "reset option!\n", arg);
                         exit(EXIT_FAILURE);
                 }
                 selfn_l3cdp_config = patterns[i].cdp;
         }
-        sel_reset_CAT = 1;
+        sel_reset_alloc = 1;
 }
 
 /**
@@ -322,6 +339,28 @@ static void selfn_show_allocation(const char *arg)
         sel_show_allocation_config = 1;
 }
 
+/**
+ * @brief Selects displaying supported capabilities
+ *
+ * @param arg not used
+ */
+static void selfn_display(const char *arg)
+{
+        UNUSED_ARG(arg);
+        sel_display = 1;
+}
+
+/**
+ * @brief Selects displaying supported capabilities in verbose mode
+ *
+ * @param arg not used
+ */
+static void selfn_display_verbose(const char *arg)
+{
+        UNUSED_ARG(arg);
+        sel_display_verbose = 1;
+}
+
 /**
  * @brief Selects allocation profile from internal DB
  *
@@ -333,6 +372,18 @@ selfn_allocation_select(const char *arg)
         selfn_strdup(&sel_allocation_profile, arg);
 }
 
+/**
+ * @brief Selects library OS interface
+ *
+ * @param arg not used
+ */
+static void
+selfn_iface_os(const char *arg)
+{
+        UNUSED_ARG(arg);
+        sel_interface = PQOS_INTER_OS;
+}
+
 /**
  * @brief Opens configuration file and parses its contents
  *
@@ -349,6 +400,8 @@ parse_config_file(const char *fname)
                 void (*fn)(const char *);
         } optab[] = {
                 {"show-alloc:",         selfn_show_allocation },   /**< -s */
+                {"display:",            selfn_display },           /**< -d */
+                {"display-verbose:",    selfn_display_verbose },   /**< -D */
                 {"log-file:",           selfn_log_file },          /**< -l */
                 {"verbose-mode:",       selfn_verbose_mode },      /**< -v */
                 {"super-verbose-mode:", selfn_super_verbose_mode },/**< -V */
@@ -362,7 +415,8 @@ parse_config_file(const char *fname)
                 {"monitor-file:",       selfn_monitor_file },      /**< -o */
                 {"monitor-file-type:",  selfn_monitor_file_type }, /**< -u */
                 {"monitor-top-like:",   selfn_monitor_top_like },  /**< -T */
-                {"reset-cat:",          selfn_reset_cat },         /**< -R */
+                {"reset-cat:",          selfn_reset_alloc },       /**< -R */
+                {"iface-os:",           selfn_iface_os },          /**< -I */
         };
         FILE *fp = NULL;
         char cb[256];
@@ -429,10 +483,11 @@ parse_config_file(const char *fname)
 static const char *m_cmd_name = "pqos";                     /**< command name */
 static const char help_printf_short[] =
         "Usage: %s [-h] [--help] [-v] [--verbose] [-V] [--super-verbose]\n"
-        "          [-l FILE] [--log-file=FILE]\n"
+        "          [-l FILE] [--log-file=FILE] [-I] [--iface-os]\n"
         "       %s [-s] [--show]\n"
-        "       %s [-m EVTCORES] [--mon-core=EVTCORES] | [-p EVTPIDS] "
-        "[--mon-pid=EVTPIDS]\n"
+        "       %s [-d] [--display] [-D] [--display-verbose]\n"
+        "       %s [-m EVTCORES] [--mon-core=EVTCORES] | [-p [EVTPIDS]] "
+        "[--mon-pid[=EVTPIDS]]\n"
         "          [-t SECONDS] [--mon-time=SECONDS]\n"
         "          [-i N] [--mon-interval=N]\n"
         "          [-T] [--mon-top]\n"
@@ -440,7 +495,7 @@ static const char help_printf_short[] =
         "          [-u TYPE] [--mon-file-type=TYPE]\n"
         "          [-r] [--mon-reset]\n"
         "       %s [-e CLASSDEF] [--alloc-class=CLASSDEF]\n"
-        "          [-a CLASS2CORE] [--alloc-assoc=CLASS2CORE]\n"
+        "          [-a CLASS2ID] [--alloc-assoc=CLASS2ID]\n"
         "       %s [-R] [--alloc-reset]\n"
         "       %s [-H] [--profile-list] | [-c PROFILE] "
         "[--profile-set=PROFILE]\n"
@@ -452,20 +507,26 @@ static const char help_printf_long[] =
         "  -v, --verbose               verbose mode\n"
         "  -V, --super-verbose         super-verbose mode\n"
         "  -s, --show                  show current PQoS configuration\n"
+        "  -d, --display               display supported capabilities\n"
+        "  -D, --display-verbose       display supported capabilities in verbose mode\n"
         "  -f FILE, --config-file=FILE load commands from selected file\n"
         "  -l FILE, --log-file=FILE    log messages into selected file\n"
         "  -e CLASSDEF, --alloc-class=CLASSDEF\n"
         "          define allocation classes.\n"
         "          CLASSDEF format is 'TYPE:ID=DEFINITION;'.\n"
-        "          To specify sockets 'TYPE[@SOCK_LIST]:ID=DEFINITION;'.\n"
-        "          Example: 'llc:0=0xffff;llc:1=0x00ff;llc@0-1:2=0xff00',\n"
-	"                   'llc:0d=0xfff;llc:0c=0xfff00;l2:2=0x3f;l2@2:1=0xf'.\n"
-        "  -a CLASS2CORE, --alloc-assoc=CLASS2CORE\n"
-        "          associate cores with an allocation class.\n"
-        "          CLASS2CORE format is 'TYPE:ID=CORE_LIST'.\n"
+        "          To specify specific resources 'TYPE[@RESOURCE_ID]:ID=DEFINITION;'.\n"
+        "          Examples: 'llc:0=0xffff;llc:1=0x00ff;llc@0-1:2=0xff00',\n"
+	"                    'llc:0d=0xfff;llc:0c=0xfff00',\n"
+        "                    'l2:2=0x3f;l2@2:1=0xf',\n"
+        "                    'mba:1=30;mba@1:3=80'.\n"
+        "  -a CLASS2ID, --alloc-assoc=CLASS2ID\n"
+        "          associate cores/tasks with an allocation class.\n"
+        "          CLASS2ID format is 'TYPE:ID=CORE_LIST/TASK_LIST'.\n"
         "          Example 'llc:0=0,2,4,6-10;llc:1=1'.\n"
+        "          Example 'core:0=0,2,4,6-10;core:1=1'.\n"
+        "          Example 'pid:0=3543,7643,4556;pid:1=7644'.\n"
         "  -R [CONFIG], --alloc-reset[=CONFIG]\n"
-        "          reset allocation configuration (L2/L3 CAT)\n"
+        "          reset allocation configuration (L2/L3 CAT & MBA)\n"
         "          CONFIG can be: l3cdp-on, l3cdp-off or l3cdp-any (default).\n"
         "  -m EVTCORES, --mon-core=EVTCORES\n"
         "          select cores and events for monitoring.\n"
@@ -473,8 +534,9 @@ static const char help_printf_long[] =
         "          Example: \"all:0,2,4-10;llc:1,3;mbr:11-12\".\n"
         "          Cores can be grouped by enclosing them in square brackets,\n"
         "          example: \"llc:[0-3];all:[4,5,6];mbr:[0-3],7,8\".\n"
-        "  -p EVTPIDS, --mon-pid=EVTPIDS\n"
-        "          select process ids and events to monitor.\n"
+        "  -p [EVTPIDS], --mon-pid[=EVTPIDS]\n"
+        "          select top 10 most active (CPU utilizing) process ids to monitor\n"
+        "          or select process ids and events to monitor.\n"
         "          EVTPIDS format is 'EVENT:PID_LIST'.\n"
         "          Example 'llc:22,25673' or 'all:892,4588-4592'.\n"
         "          Note: processes and cores cannot be monitored together.\n"
@@ -493,7 +555,11 @@ static const char help_printf_long[] =
         "  -H, --profile-list          list supported allocation profiles\n"
         "  -c PROFILE, --profile-set=PROFILE\n"
         "          select a PROFILE of predefined allocation classes.\n"
-        "          Use -H to list available profiles.\n";
+        "          Use -H to list available profiles.\n"
+        "  -I, --iface-os\n"
+        "          set the library interface to use the kernel\n"
+        "          implementation. If not set the default implementation is\n"
+        "          to program the MSR's directly.\n";
 
 /**
  * @brief Displays help information
@@ -504,32 +570,35 @@ static const char help_printf_long[] =
 static void print_help(const int is_long)
 {
         printf(help_printf_short,
-               m_cmd_name, m_cmd_name, m_cmd_name, m_cmd_name,
+               m_cmd_name, m_cmd_name, m_cmd_name, m_cmd_name, m_cmd_name,
                m_cmd_name, m_cmd_name, m_cmd_name);
         if (is_long)
                 printf(help_printf_long);
 }
 
 static struct option long_cmd_opts[] = {
-        {"help",          no_argument,       0, 'h'},
-        {"log-file",      required_argument, 0, 'l'},
-        {"config-file",   required_argument, 0, 'f'},
-        {"show",          no_argument,       0, 's'},
-        {"profile-list",  no_argument,       0, 'H'},
-        {"profile-set",   required_argument, 0, 'c'},
-        {"mon-interval",  required_argument, 0, 'i'},
-        {"mon-pid",       required_argument, 0, 'p'},
-        {"mon-core",      required_argument, 0, 'm'},
-        {"mon-time",      required_argument, 0, 't'},
-        {"mon-top",       no_argument,       0, 'T'},
-        {"mon-file",      required_argument, 0, 'o'},
-        {"mon-file-type", required_argument, 0, 'u'},
-        {"mon-reset",     no_argument,       0, 'r'},
-        {"alloc-class",   required_argument, 0, 'e'},
-        {"alloc-reset",   required_argument, 0, 'R'},
-        {"alloc-assoc",   required_argument, 0, 'a'},
-        {"verbose",       no_argument,       0, 'v'},
-        {"super-verbose", no_argument,       0, 'V'},
+        {"help",            no_argument,       0, 'h'},
+        {"log-file",        required_argument, 0, 'l'},
+        {"config-file",     required_argument, 0, 'f'},
+        {"show",            no_argument,       0, 's'},
+        {"display",         no_argument,       0, 'd'},
+        {"display-verbose", no_argument,       0, 'D'},
+        {"profile-list",    no_argument,       0, 'H'},
+        {"profile-set",     required_argument, 0, 'c'},
+        {"mon-interval",    required_argument, 0, 'i'},
+        {"mon-pid",         required_argument, 0, 'p'},
+        {"mon-core",        required_argument, 0, 'm'},
+        {"mon-time",        required_argument, 0, 't'},
+        {"mon-top",         no_argument,       0, 'T'},
+        {"mon-file",        required_argument, 0, 'o'},
+        {"mon-file-type",   required_argument, 0, 'u'},
+        {"mon-reset",       no_argument,       0, 'r'},
+        {"alloc-class",     required_argument, 0, 'e'},
+        {"alloc-reset",     required_argument, 0, 'R'},
+        {"alloc-assoc",     required_argument, 0, 'a'},
+        {"verbose",         no_argument,       0, 'v'},
+        {"super-verbose",   no_argument,       0, 'V'},
+        {"iface-os",        no_argument,       0, 'I'},
         {0, 0, 0, 0} /* end */
 };
 
@@ -538,17 +607,19 @@ int main(int argc, char **argv)
         struct pqos_config cfg;
         const struct pqos_cpuinfo *p_cpu = NULL;
         const struct pqos_cap *p_cap = NULL;
-        const struct pqos_capability *cap_mon = NULL,
-		*cap_l3ca = NULL, *cap_l2ca = NULL;
+        const struct pqos_capability *cap_mon = NULL, *cap_l3ca = NULL,
+                *cap_l2ca = NULL, *cap_mba = NULL;
         unsigned sock_count, *sockets = NULL;
         int cmd, ret, exit_val = EXIT_SUCCESS;
-        int opt_index = 0;
+        int opt_index = 0, pid_flag = 0;
 
         m_cmd_name = argv[0];
         print_warning();
 
+        memset(&cfg, 0, sizeof(cfg));
+
         while ((cmd = getopt_long(argc, argv,
-                                  ":Hhf:i:m:Tt:l:o:u:e:c:a:p:srvVR:",
+                                  ":Hhf:i:m:Tt:l:o:u:e:c:a:p:sdDrvVIR:",
                                   long_cmd_opts, &opt_index)) != -1) {
                 switch (cmd) {
                 case 'h':
@@ -570,7 +641,21 @@ int main(int argc, char **argv)
                         selfn_monitor_interval(optarg);
                         break;
                 case 'p':
-		        selfn_monitor_pids(optarg);
+                        if (optarg != NULL && *optarg == '-') {
+                                /**
+                                 * Next switch option wrongly assumed to be
+                                 * argument to '-p'.
+                                 * In order to fix it, we are handling this as
+                                 * '-p' without parameters (as it should be)
+                                 * to start top-pids monitoring mode.
+                                 * Have to rewind \a optind as well.
+                                 */
+                                selfn_monitor_top_pids();
+                                optind--;
+                                break;
+                        }
+                        selfn_monitor_pids(optarg);
+                        pid_flag = 1;
                         break;
                 case 'm':
                         selfn_monitor_cores(optarg);
@@ -597,36 +682,47 @@ int main(int argc, char **argv)
                         sel_mon_reset = 1;
                         break;
                 case 'R':
-                        if (optarg != NULL && *optarg == '-') {
-                                /**
-                                 * Next switch option wrongly assumed to be
-                                 * argument to '-R'.
-                                 * Pass NULL as argument to '-R' function and
-                                 * rewind \a optind.
-                                 */
-                                selfn_reset_cat(NULL);
-                                optind--;
-                        } else {
-                                selfn_reset_cat(optarg);
-                        }
+                        if (optarg != NULL) {
+                                if (*optarg == '-') {
+                                        /**
+                                        * Next switch option wrongly assumed
+                                        * to be argument to '-R'.
+                                        * Pass NULL as argument to '-R' function
+                                        * and rewind \a optind.
+                                        */
+                                        selfn_reset_alloc(NULL);
+                                        optind--;
+                                } else
+                                        selfn_reset_alloc(optarg);
+                        } else
+                                selfn_reset_alloc(NULL);
                         break;
                 case ':':
                         /**
                          * This is handler for missing mandatory argument
                          * (enabled by leading ':' in getopt() argument).
-                         * -R is only allowed switch for optional argument.
+                         * -R and -p are only allowed switch for optional args.
                          * Other switches need to report error.
                          */
-                        if (optopt != 'R') {
+                        if (optopt == 'R') {
+                                selfn_reset_alloc(NULL);
+                        } else if (optopt == 'p') {
+                                /**
+                                 * Top pids mode - in case of '-I -p' top N
+                                 * pids (by CPU usage) will be displayed and
+                                 * monitored for cache/mbm/misses
+                                 */
+                                selfn_monitor_top_pids();
+                                pid_flag = 1;
+                        } else {
                                 printf("Option -%c is missing required "
                                        "argument\n", optopt);
                                 return EXIT_FAILURE;
-                        } else {
-                                selfn_reset_cat(NULL);
                         }
                         break;
                 case 'a':
                         selfn_allocation_assoc(optarg);
+                        pid_flag |= alloc_pid_flag;
                         break;
                 case 'c':
                         selfn_allocation_select(optarg);
@@ -634,12 +730,21 @@ int main(int argc, char **argv)
                 case 's':
                         selfn_show_allocation(NULL);
                         break;
+                case 'd':
+                        selfn_display(NULL);
+                        break;
+                case 'D':
+                        selfn_display_verbose(NULL);
+                        break;
                 case 'v':
                         selfn_verbose_mode(NULL);
                         break;
                 case 'V':
                         selfn_super_verbose_mode(NULL);
                         break;
+                case 'I':
+                        selfn_iface_os(NULL);
+                        break;
                 default:
                         printf("Unsupported option: -%c. "
                                "See option -h for help.\n", optopt);
@@ -652,9 +757,14 @@ int main(int argc, char **argv)
                 }
         }
 
-        memset(&cfg, 0, sizeof(cfg));
+        if (pid_flag == 1 && sel_interface == PQOS_INTER_MSR) {
+                printf("Error! OS interface option [-I] needed for PID"
+                       " operations. Please re-run with the -I option.\n");
+                exit_val = EXIT_FAILURE;
+                goto error_exit_1;
+        }
         cfg.verbose = sel_verbose_mode;
-
+        cfg.interface = sel_interface;
         /**
          * Set up file descriptor for message log
          */
@@ -666,7 +776,7 @@ int main(int argc, char **argv)
                 if (cfg.fd_log == -1) {
                         printf("Error opening %s log file!\n", sel_log_file);
                         exit_val = EXIT_FAILURE;
-                        goto error_exit_2;
+                        goto error_exit_1;
                 }
         }
 
@@ -712,6 +822,13 @@ int main(int argc, char **argv)
                 goto error_exit_2;
         }
 
+        ret = pqos_cap_get_type(p_cap, PQOS_CAP_TYPE_MBA, &cap_mba);
+        if (ret == PQOS_RETVAL_PARAM) {
+                printf("Error retrieving MB allocation capabilities!\n");
+                exit_val = EXIT_FAILURE;
+                goto error_exit_2;
+        }
+
         if (sel_mon_reset && cap_mon != NULL) {
                 if (pqos_mon_reset() != PQOS_RETVAL_OK) {
                         exit_val = EXIT_FAILURE;
@@ -721,23 +838,32 @@ int main(int argc, char **argv)
                 }
         }
 
-        if (sel_reset_CAT) {
+        if (sel_reset_alloc) {
                 /**
-                 * Reset CAT configuration to after-reset state and exit
+                 * Reset allocation configuration to after-reset state and exit
                  */
                 if (pqos_alloc_reset(selfn_l3cdp_config) != PQOS_RETVAL_OK) {
                         exit_val = EXIT_FAILURE;
-                        printf("CAT reset failed!\n");
+                        printf("Allocation reset failed!\n");
                 } else
-                        printf("CAT reset successful\n");
+                        printf("Allocation reset successful\n");
         }
 
         if (sel_show_allocation_config) {
                 /**
                  * Show info about allocation config and exit
                  */
-		alloc_print_config(cap_mon, cap_l3ca, cap_l2ca, sock_count,
-                                   sockets, p_cpu);
+		alloc_print_config(cap_mon, cap_l3ca, cap_l2ca, cap_mba,
+                                   sock_count, sockets, p_cpu,
+                                   sel_verbose_mode);
+                goto allocation_exit;
+        }
+
+        if (sel_display || sel_display_verbose) {
+                /**
+                 * Display info about supported capabilities
+                 */
+                cap_print_features(p_cap, p_cpu, sel_display_verbose);
                 goto allocation_exit;
         }
 
@@ -748,7 +874,7 @@ int main(int argc, char **argv)
                 }
         }
 
-        switch (alloc_apply(cap_l3ca, cap_l2ca, sock_count, sockets)) {
+        switch (alloc_apply(cap_l3ca, cap_l2ca, cap_mba, p_cpu)) {
         case 0: /* nothing to apply */
                 break;
         case 1: /* new allocation config applied and all is good */
@@ -764,7 +890,7 @@ int main(int argc, char **argv)
         /**
          * If -R was present ignore all monitoring related options
          */
-        if (sel_reset_CAT)
+        if (sel_reset_alloc)
                 goto allocation_exit;
 
         /**
